#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import print_function

import sys
import os.path
import datetime
import xml.etree.ElementTree as ET
import zipfile
from pyspatialite import dbapi2 as db


NS = '{http://www.topografix.com/GPX/1/1}'


def _in_time(fun):
    def _fun(*args, **kwargs):
        start = datetime.datetime.now()
        fun(*args, **kwargs)
        finish = datetime.datetime.now()
        print('in', finish - start, file=sys.stderr)
    return _fun


@_in_time
def _create_schema(cursor):
    print('creating schema ...', file=sys.stderr)

    cursor.execute('SELECT InitSpatialMetadata(1)')     # transaction=1 means to run it on one transaction

    cursor.execute('''
CREATE TABLE track (
    id_track INTEGER PRIMARY KEY,
    id_driver INTEGER,
    start_dt TEXT,
    finish_dt TEXT
)''')
    _add_geometry_column(cursor, 'track', 'LINESTRING')
    cursor.execute('''
CREATE TABLE point (
    id_point INTEGER PRIMARY KEY,
    id_track INTEGER REFERENCES track (id_track),
    ele REAL,
    time TEXT,
    sat INTEGER,
    hdop REAL,
    vdop REAL,
    course REAL,
    speed REAL
)''')
    _add_geometry_column(cursor, 'point', 'POINT')

    print('created schema', file=sys.stderr)


def _add_geometry_column(cursor, table_name, geo_type='MULTIPOLYGON'):
    cursor.execute('''
SELECT AddGeometryColumn(
    '%s',
    'geom',
    4326,
    '%s',
    2
)''' % (table_name, geo_type))
    cursor.execute('''
SELECT CreateSpatialIndex(
    '%s', 'geom'
)''' % table_name)


@_in_time
def _insert_track(track_file, cursor):
    gpx = ET.parse(track_file).getroot()
    metadata = gpx.find(NS + 'metadata')
    track_id = metadata.find(NS + 'name').text
    driver = metadata.find(NS + 'driver').text
    print('inserting track', track_id, '...', file=sys.stderr, end=' ')

    cursor.execute('''INSERT OR REPLACE INTO track (id_track, id_driver) VALUES (?, ?)''',
        (track_id, driver))
    cursor.execute('''DELETE FROM point WHERE id_track = ?''', (track_id,))

    start_dt = '9999'
    finish_dt = '0000'
    points_geom = []
    for point in gpx.find(NS + 'trk').find(NS + 'trkseg').findall(NS + 'trkpt'):
        point_geom = '%.8f %.8f' % (float(point.get('lon')), float(point.get('lat')))
        points_geom.append(point_geom)
        point_wkt = 'POINT (' + point_geom + ')'
        time = point.find(NS + 'time').text
        if time < start_dt:
            start_dt = time
        if time > finish_dt:
            finish_dt = time
        cursor.execute('''INSERT INTO point (id_track, ele, time, sat, hdop, vdop, course, speed, geom)
            VALUES (?, ?, ?, ?, ?, ?, ?, ?, GeomFromText(?, 4326))''',
            (track_id,
             point.find(NS + 'ele').text, time, point.find(NS + 'sat').text,
             point.find(NS + 'hdop').text, point.find(NS + 'vdop').text,
             point.find(NS + 'course').text, point.find(NS + 'speed').text,
             point_wkt))

    track_wkt = 'LINESTRING (' + ', '.join(points_geom) + ')'
    cursor.execute('''UPDATE track SET start_dt = ?, finish_dt = ?, geom = GeomFromText(?, 4326)
        WHERE id_track = ?''',
        (start_dt, finish_dt, track_wkt, track_id))


@_in_time
def _read_zip(zip_filename, cursor, connection):
    print('reading', zip_filename, '...', file=sys.stderr)
    with zipfile.ZipFile(zip_filename) as zip:
        for info in zip.infolist():
            if info.file_size == 0:
                continue
            with zip.open(info.filename) as gpx_file:
                _insert_track(gpx_file, cursor)
    connection.commit()
    print('processed', zip_filename, file=sys.stderr, end=' ')


def main(filenames):
    sqlite_filename = filenames[-1]
    new_sqlite = not os.path.exists(sqlite_filename)
    gpx_filenames = []
    zip_filenames = []
    for filename in filenames[:-1]:
        if filename.lower().endswith('.gpx'):
            gpx_filenames.append(filename)
        elif filename.lower().endswith('.zip'):
            zip_filenames.append(filename)

    connection = db.connect(sqlite_filename)
    cursor = connection.cursor()
    print('writing', sqlite_filename, '...', file=sys.stderr)

    if new_sqlite:
        _create_schema(cursor)

    for filename in gpx_filenames:
        with open(filename, 'rt') as file:
            _insert_track(file, cursor)
        connection.commit()

    for filename in zip_filenames:
        _read_zip(filename, cursor, connection)

    cursor.close()
    connection.close()


if __name__ == '__main__':

    if len(sys.argv) < 3:
        print('Usage: gpx2splite.py ( file.gpx | file.zip )+  file.sqlite')
        sys.exit(1)

    main(sys.argv[1:])
